### A Pluto.jl notebook ###
# v0.19.40

using Markdown
using InteractiveUtils

# ╔═╡ a145e2fe-a903-11eb-160f-df7ea83fa3e6
begin
	using PlotlyBase, HypertextLiteral, PlutoUI, PlutoTeachingTools, PlutoPlotly
	using CSV, DataFrames, Dates, PeriodicalDates, StatsBase
	#import PlotlyJS:savefig
end

# ╔═╡ ee9154d6-739a-46d6-aa1c-0467d4b49f5a
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 943edb36-acc8-4b2a-94ab-c544f9eb279b
md"""

# Week 6 - The Central Bank & Monetary Policy Tools

### Exercises

**Macroeconomics, ISCTE-IUL**

"""

# ╔═╡ 1245aef9-18ce-42f5-af82-c2e6cce70798
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas;  March 2024**
"""

# ╔═╡ 4618a663-f77e-4ccc-9b00-e84dbd5beb34
md"""

### Packages

"""

# ╔═╡ 74de9f66-d5b4-4e0f-a75d-5177b1842191
begin
	function Base.show(io::IO, mimetype::MIME"text/html", plt::PlotlyBase.Plot)
       # Remove responsive flag on the plot as we handle responsibity via ResizeObeserver and leaving it on makes the div flickr during updates
	hasproperty(plt,:config) && plt.config.responsive && (plt.config.responsive = false)   
	show(io,mimetype, @htl("""
			<div>
			<script id=asdf>
			const {plotly} = await import("https://cdn.plot.ly/plotly-2.2.0.min.js")
			const PLOT = this ?? document.createElement("div");
		

		
			Plotly.react(PLOT, $(HypertextLiteral.JavaScript(PlotlyBase.json(plt))));


		
			const pluto_output = currentScript.parentElement.closest('pluto-output')

			const resizeObserver = new ResizeObserver(entries => {
				Plotly.Plots.resize(PLOT)
			})

			resizeObserver.observe(pluto_output)

			invalidation.then(() => {
				resizeObserver.disconnect()
			})
		
			return PLOT
			</script>
			</div>
	"""))
	end
end

# ╔═╡ 72d45107-22d6-4663-a9b8-c540a4c7411b
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 116306a9-5f2d-4b96-9124-584dc9835a8a
md"""
## Exercise 1. The central bank´s balance sheet
"""

# ╔═╡ 9877083b-a80c-4069-89e4-84638fe6ecbb
md"""
Using Tables 1.1 and 1.2, which cover the balance sheet of the Fed for the years 2007 and 2009, answer the following questions:

👉 a) What main monetary aggregate is missing from the data?

👉 b) Calculate the value of `Foreign reserves` and `Securities`.

👉 c) Complete the balance sheets according to the structure provided in the slides.

👉 d) The data from 2007 reflects the Fed's balance sheet before the financial crisis blew up in 2008. At the end of 2008, the Fed started a type of policy known as "Quantitative Easing", and this new policy is already visible in the balance sheet of 2009. What are the main differences between the two years? 

"""

# ╔═╡ 0be26b61-30ce-4128-b2b1-b4eb955244d1
aside(hint(md"```math
\begin{aligned}
&\text { Assets - Central Bank }\\
&\begin{array}{lr}
\hline \text { Items } & \$ \text { billions } \\
\hline \text { Foreign Reserves } & 0 \\
\text { Securities } & 0 \\
\text { Loans } & 0 \\
\text { Other assets } & 0 \\
\text { Total assets } & 0
\end{array}
\end{aligned}
```") ,v_offset=60)

# ╔═╡ cfc72d68-3d45-4673-81a2-7032896fbe89
md"""
$$\begin{aligned}
		&\text { Table 1.1$-$Federal Reserve Board: Assets (\$ billions) }\\
		&\begin{array}{llrr}
			\hline \hline \text {Code} & \text {Items} & \text {2009} & \text {2007} \\
			\hline 
			{tre} & \text {Treasuries*}                 & \qquad \quad  776 & \qquad \quad 797 \\
			{mbs} & \text {Mortage-backed securities*}  & \qquad \quad  801 & \qquad \quad   0 \\
			{fad} & \text {Federal Agency debt*}        & \qquad \quad  258 & \qquad \quad   0 \\
			{oa} & \text {Other Assets}                & \qquad \quad   91 & \qquad \quad  14 \\
			{gol} & \text {Gold}                        & \qquad \quad   12 & \qquad \quad  11 \\
			{fa} & \text {Foreign Assets}            & \qquad \quad   25 & \qquad \quad  47 \\
			{loa} & \text {Loans}                       & \qquad \quad  276 & \qquad \quad  25 \\
			ass & \text {Total Assets}                & \qquad \quad 2239 & \qquad \quad 894 \\
			\hline
\end{array}
\end{aligned}$$
$^* \text{All these items are Securities}$
"""

# ╔═╡ be5df6ed-9417-4ed0-862c-8f337b80084e
aside(hint(md"```math
\begin{aligned}
&\text { Liabilities - Central Bank }\\
&\begin{array}{lr}
\hline \text { Items } & \$ \text { billions } \\
\hline \text { Currency } & 0 \\
\text { Reserves } & 0 \\
\text { Government account } & 0 \\
\text { Other liabilities } & 0 \\
\text { Equity } & 0 \\
\text { Liability + Equity } & 0
\end{array}
\end{aligned}
```") ,v_offset=0)

# ╔═╡ f0a318ef-47b0-4fa3-aeb1-e7d1548fd214
md"""
$$\begin{aligned}
		&\text { Table 1.2$-$Federal Reserve Board: Liabilities (\$ billions)}\\
		&\begin{array}{llrr}
			\hline \hline \text {Code} & \text {Items} & \text {2009} & \text {2007} \\
			\hline 
			cur & \text {Currency}                     & \qquad \qquad  883 & \qquad \quad 792 \\
			gov & \text {Government Account}           & \qquad \qquad  149 & \qquad \quad   5 \\
			res & \text {Reserves of commercial banks} & \qquad \qquad 1073 & \qquad \quad  11 \\
			ol & \text {Other liabilities}            & \qquad \qquad   81 & \qquad \quad  49 \\
			lia &\text {Total liabilities}            & \qquad \qquad 2187 & \qquad \quad 857 \\
			\hline
		\end{array}
	\end{aligned}$$
"""

# ╔═╡ b189ebff-1493-4d95-98d2-99d6f55884fe
md"""

We will pass to the notebook only the information concerning 2009 (the index `_9` is used to refer to 2009 data) . It would help if you tried to do the same thing for the year 2007.

"""

# ╔═╡ c99c90e1-e91f-4682-a190-495d2cbd0b06
tre_9 = 776 ; mbs_9 = 801 ; fad_9 = 258 ; oa_9 = 91 ; gol_9 = 12 ; fa_9 = 25 ; loa_9 = 276 ; ass_9 = 2239 ;

# ╔═╡ 98e7e1ad-2964-41e5-9162-cd077978ee15
cur_9 = 883 ; gov_9 = 149 ; res_9 = 1073 ; ol_9 = 81 ; lia_9 = 2187 ;

# ╔═╡ 719dbca7-f3f1-4827-bfba-d9813979c42a
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ f83dd4f0-0312-4bb0-98b0-67eeab7177e4
md"""
!!! hint

	To complete the balance sheet, we must calculate the total amounts of `Foreign Reserves` and `Securities`. 

	We will do it only for the year 2009. Students are advised to do the same for 2007.

"""

# ╔═╡ 11d60c86-9825-4113-883f-66c82bcb29e9
fr_9 = fa_9 + gol_9

# ╔═╡ bc07b610-26f4-4577-b7cb-0089f08b7a60
sec_9 = tre_9 + mbs_9 + fad_9 

# ╔═╡ 02126494-1ba6-47f8-8c10-9c4d34e20d57
md"""
!!! answer "Answer (b)"

	Here

	Here

"""

# ╔═╡ 84cee9af-0999-4659-a537-85c89f9182cc
md"""
!!! answer "Answer (c)"

	Here

"""

# ╔═╡ c144c3fe-d49f-4243-bdc2-34ba8af88673
md"""
!!! answer "Answer (d)"

	Here

"""

# ╔═╡ 1a073295-059c-4815-b3bd-450f5f88c8d1
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 99fa722b-bef4-48d7-bd4a-f0459f15eed3
md"""
## Exercise 2. Fed operations and the Monetary Base
"""

# ╔═╡ f4706381-35df-44dd-aea2-83220a94c9d4
md"""

Write down in the balance sheet of the central bank the following operations:

👉 (1) The Fed sells 200 million dollars of bonds denominated in Renminbi.

👉 (2) The Fed buys 600 million of Mortgage-Backed Securities.

👉 (3) The Fed buys 900 million of 10-Year Treasury bonds.

👉 (4) The Fed lends 500 million to commercial banks.

👉 (5) Due to an unexpected event, the private sector withdraws 200 million from bank accounts.

👉 (6) The Fed decided to raise the IOER to 0.25 percentage points.

👉 (7) The Fed buys 400 million dollars of bonds denominated in Euros.

"""

# ╔═╡ 8eeea220-e319-45da-a774-7cf1475f2424
aside(danger(md"The Fed sells 200 million dollars of bonds denominated in Renminbi."),v_offset=100)

# ╔═╡ 4f82c4f5-dab7-4ef4-a1c6-bef80ac2fe64
md"""
!!! answer "Answer (1)"

	The Fed's balance sheet will change as follows:

	Here

	The comercial banks' consolidadted balance sheet will change as follows:

	Here

"""

# ╔═╡ 71bbd822-914c-4ea6-93c5-0b3ad86000ca
aside(danger(md"The Fed buys 600 million of Mortgage-Backed Securities."),v_offset=100)

# ╔═╡ 10234ea1-6528-4fd2-bdae-3ca5a36e405b
md"""
!!! answer "Answer (2)"

	The Fed's balance sheet will change as follows:

	Here

	The commercial banks' consolidated balance sheet will change as follows:

	Here

"""

# ╔═╡ 76d208b1-80b7-4a8c-96c6-2de15b5e89e8
aside(danger(md"The Fed buys 900 million of 10-Year Treasury bonds."),v_offset=45)

# ╔═╡ 1254cc13-8ac0-468c-bbc5-e838018fa1de
md"""
!!! answer "Answer (3)"

	The Fed's balance sheet will change as follows:

	Here

	The commercial banks' consolidated balance sheet will change as follows:

	Here

"""

# ╔═╡ e7f794fa-ede6-4391-b703-5488fd739f29
aside(danger(md"The Fed lends 500 million to commercial banks."),v_offset=100)

# ╔═╡ 15b3a15f-a0a0-4623-b6ec-43dc33bbe1e3
md"""
!!! answer "Answer (4)"

	The Fed's balance sheet will change as follows:

	Here  	
	

	The commercial banks' consolidated balance sheet will change as follows:

	Here

"""

# ╔═╡ d2a12416-9d54-479c-956d-2d8869024727
aside(danger(md"The private sector withdraws 200 million from bank accounts."),v_offset=100)

# ╔═╡ 3323975f-e90c-4a39-87d4-2ae9c676e24a
md"""
!!! answer "Answer (5)"

	The Fed's balance sheet will change as follows:

	Here

	The commercial banks' consolidated balance sheet will change as follows:

	Here

"""

# ╔═╡ a4aecd27-1376-4f74-98b6-10502c93e1d0
aside(danger(md"The Fed decided to raise the IOER to 0.25 percentage points."),v_offset=50)

# ╔═╡ d865e28a-5db4-4b39-a7e6-353d380b9e1a
md"""
!!! answer "Answer (6)"

	Here

"""

# ╔═╡ 32c928dd-ca87-4db5-aa28-222e43a7a3cb
aside(danger(md"The Fed buys 400 million dollars of bonds denominated in Euros."),v_offset=50)

# ╔═╡ 5d7ad15e-3057-43c8-913c-67f6a71338b7
md"""
!!! answer "Answer (7)"

	Here

"""

# ╔═╡ 821afa78-c1c7-4f08-a1a4-0ae7682c1c2b
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ c842b2da-1bf5-43db-9bc0-a8af8be64b9c
md"""
## Exercise 3. Monetary Base vs Total Assets
"""

# ╔═╡ 7e7e4292-dfc8-41b4-a5a0-d7646f8fb495
md"""

In the following figure, we plot the evolution of two important monetary aggregates for the US economy  -- `Total Assets` and the `Monetary Base` (MB) -- between 2003 and August 2022. By inspecting the plot (use the zoom facility), analyze the behavior of these two crucial macroeconomic aggregates in three different periods (before 2008, between 2008 and 2019, and after 2019), considering the gap between these two aggregates.

"""

# ╔═╡ e547d364-216c-4165-982f-1e04c9a0945c
md"""
!!! answer "Suggested answer"

	Here

	Here

	Here

"""

# ╔═╡ f54d0589-0567-4b75-91f2-d9bf223c5fb6
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 2bd8b31a-8fa2-4404-816e-158df0afcc7d
md"""
## Exercise 4. The Money Multiplier
"""

# ╔═╡ e7fd66aa-cb38-4b7f-92af-aa4fd15549b2
md"""
Using the data file "Multiplier.csv", we plot the Monetary Base (MB) and the Money Supply (M2) series for the USA economy between 1959 and 2022. 

👉 a) What is the money multiplier?

👉 b) Looking at the first plot below, what can we conclude about the relationship between M2 and the MB over the period under consideration?

👉 c) In the second figure below, we plot the money multiplier. Does this multiplier display a stable behavior?

👉 d) What is the main implication for managing monetary policy that arises from large instability in the money multiplier?


"""

# ╔═╡ cd8e9140-5696-4019-b208-d6430e68f978
MM = CSV.read("Multiplier.csv", delim=";", DataFrame)

# ╔═╡ f8a3949e-aae6-471f-8fb4-d536566d6e8a
period5_1 = MonthlyDate(1959,1):Month(1):MonthlyDate(2022,8);

# ╔═╡ c5363210-26f9-40b1-9a1a-9ac80ffac890
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ a9095027-f033-4253-9269-105e51887424
begin
	
fig4_1 = Plot(Date.(period5_1), [MM[:,2] MM[:,3]])

relayout!(fig4_1, Layout(hovermode="x", title_text="MB vs M2 in the USA (1959--2022): Billions of Dollars", title_x = 0.5, titlefont_size="17"))

restyle!(fig4_1, 1:2, name = ["M2", "MB"], line_color = ["Blue" , "red"], line_width="2.5")
	
fig4_1

end

# ╔═╡ 1e9a9875-a58e-4e4c-add2-9d5cd667384f
begin
	trace6_5 = scatter(;x = Date.(period5_1), y = MM[:,2], name = "M2", line_color = "Blue", 
						mode = "lines", line_width="2.5", marker_size ="6")
	
	
	trace6_6 = scatter(;x = Date.(period5_1), y = MM[:,3], name = "MB", line_color = "red", 
						mode = "lines", line_width="2.5", marker_size ="4", 
						yaxis = "y2")
	
	
	layout6_6 = Layout(
		
			title_text = "MB vs M2 in the USA (1959--2022): Billions of Dollars", title_x=0.5,
		
			hovermode="x",

			legend=attr( x=0.05, y=0.95),
		
			xaxis = attr(
                		title = "Monthly obervations",
                		tickformat = "%Y",
                		#hoverformat = "%Y-M%m",
                		#tick0 = "1989/19/01",
                		#dtick = "M48"
        						),
		
			yaxis1 = attr(
						title = "M2",
						titlefont_color=  "blue",
						tickfont_color = "blue",
						#overlaying = "y",
						#side = "right",
						#yaxis1_tickvals = [1.5, 2.5, 3.5, 5.5],
						showgrid= false,
						zeroline=false,
						#yaxis1_range=[1.5 , 5.5],
    					#dtick = 5.6 / 7
						),
	

	  		yaxis2 = attr(
						title = "MB",
						titlefont_color=  "red",
						tickfont_color = "red",
						overlaying = "y",
						side = "right",
						showgrid= false,
						zeroline=false,
						#yaxis2_range=[-27 , 27],
    					#dtick = 27 / 3
	  					)
		
						)

	p6_6 = Plot([trace6_5,trace6_6],layout6_6)
end;

# ╔═╡ c11ab4d4-281e-435b-9ba6-35ef7f4d24b9
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ 1471566f-469f-421b-8671-abc49e040bc1
begin
	trace6_7 = scatter(;x = Date.(period5_1), y = MM[:,2]./MM[:,3], 
				name = "M2", mode="lines",
				marker_symbol="circle", marker_size="4",line_width= 2.2, 
            	marker_color = "Blue")
	
	
	layout6_7 = Layout(;
		
			title_text = "The Money Multiplier for the USA (1959--2022)", title_x=0.5,
		
			hovermode="x",		
		
            xaxis = attr(
               title = "Monthly obervations"
               #tickformat = "%Y",
               #hoverformat = "%Y-M%m",
               #tick0 = "1989/09/01",
               #dtick = "M60" 
			   ),
		
        	#xaxis_range = [2000, 2020],
        	yaxis_title = "Percentage points",
        	#yaxis_range=[-2, 2], 
        	titlefont_size = 16)

	p6_7 = Plot([trace6_7], layout6_7)
end

# ╔═╡ 624e0704-a138-4ddd-8086-078d98a84a41
md"""
!!! answer "Answer (c)"

	Here

"""

# ╔═╡ fe0e5144-02b9-4df3-84ad-3298a8703baa
md"""
!!! answer "Answer (d)"

	Here

"""

# ╔═╡ 6e20a099-8d46-4a35-83ce-e62c977a6b3d
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 65da15c3-dcad-4e38-b0aa-bf9dd456285d
md"""
## Exercise 5. Successful Fed
"""

# ╔═╡ ced280ff-d8c1-414e-8855-387727d5dfe2
Resource("https://vivaldomendes.org/images/depot/Tools.png",:width=>600)

# ╔═╡ 74529060-483d-4afe-8d58-155c5f0b4b1e
md"""

Using the figure above, explain why the Fed so successfully guarantees that short-term interest rates stay where the bank wants them to be.

"""

# ╔═╡ 5e6f355c-cb29-416f-8799-0234fbc87f1f
md"""
!!! answer

	Here

"""

# ╔═╡ f4fe404c-ca57-4682-8a51-3c08abedfb36
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 60e62656-e863-4595-9d94-4e7d0cf6d935
md"""
## Exercise 6. The Fed's dilemma
"""

# ╔═╡ 88675aac-0d69-4028-aee1-c0e0c22ab5e2
md"""

The US has been experiencing the highest inflation rate for the last forty years. The following figure presents the evolution of the two main measures of inflation (Headline inflation and Core inflation) and the Fed funds rate since 2000. "Core" inflation (which excludes energy and food prices from "Headline" inflation) is one of the preferred Fed measures of inflation.

>"We have got to get inflation behind us. I wish there were a painless way to do that. There isn't." Jerome Powel, Chair of the Fed, 21 September 2022 

What does Powel mean by saying, "I wish there were a painless way to do that"?
"""

# ╔═╡ 08bf1683-0c0d-4c52-a446-6f8be4135aed
md"""
!!! answer

	Here

"""

# ╔═╡ d08dc62f-8951-4e36-8818-c325316ece23
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ bf337699-7647-4538-85e1-ed2fd9d05269
md"""
## Exercise 7. A 1M-dollar question: how far will the Fed go?
"""

# ╔═╡ 5b1919fb-f5f5-4682-a1e0-1c767825c902
md"""

>"The Fed raised borrowing costs by three-quarters of a percentage point, their third-straight jumbo increase, taking their official interest rate to a range of 3 to 3.25 percent. But they also penciled in additional increases for the rest of this year and next, projecting that rates would reach 4.4 percent by the end of the year and climb to 4.6 percent by the end of 2023." New York Times, 21 September 2022

How far will the Fed go? This is a one-million-dollar question! The following figure presents the difference between "Core" inflation and the Fed funds rate since 2020. Did the forecast of the Fed perform well last year? Do you want to have a go and provide a forecast for next year? Will the Fed raise rates to 6% or even 6.5% next year? Will it stop there?

"""

# ╔═╡ c93efdd7-1ae9-4989-b5cb-cfdb4347086d
md"""
!!! answer 

	Here

"""

# ╔═╡ 8f643bb9-8826-421f-8590-b440c65101eb
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ dca53198-44ed-402e-8a68-401a6006d1f9
md"""
## Exercise 8. Oil prices & Fed mistakes
"""

# ╔═╡ 2b7f9399-1606-4374-abfb-abf9e7ab2e90
md"""
>"In the long history of Federal Reserve mistakes, one general error stands out. They tend to wait too long and then do too much, and, in so doing, actually accentuate rather than tame the business cycle. They appear to be well on their way to repeating this error today." David Kelly, JPMorgan Funds chief global strategist, 10 October 2022

👉 a)  Is monetary policy a science?

👉 b) In the two figures below we plot the annual growth rates of crude oil prices (Oil) and the rate of Headline inflation (CPI), since 1998. The first plot is a time series of the two growth rates, while the second one is a cross-plot of them. Do you see any link between the growth rate of oil prices and the inflation rate for the US economy for that period?

👉 c) What is the implication of oil price increases for monetary policy management?

"""

# ╔═╡ 0223f27f-532b-4b75-af02-817fe5f44ed6
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ b04f7af0-7e7c-45a5-ad82-78e866348567
md"""
!!! answer "Suggested answer (b)"

	Here

"""

# ╔═╡ 1a8a1ae4-6ecd-4a91-a482-3024ca11fb98
md"""
!!! answer "Suggested answer (c)"

	Here

"""

# ╔═╡ 69621e4b-5163-40a4-a70c-429d8afe6ff5
md"""
____________________________________________________________________________________________________________
"""

# ╔═╡ 01875faf-3231-4bb8-899d-0dd4e569ddc9
md"""
## Exercise 9. COVID19 and monetary policy
"""

# ╔═╡ 63efdb6e-dff1-420f-a99d-90cece86bac2
md"""
In Table 9.1, we present the data on August 11, 2021, of the balance sheet of the Federal Reserve Board. This data was compiled from [FRB](https://www.federalreserve.gov/releases/h41/current/h41.htm) on August 13. This table reflects the drastic intervention of the Fed in the financial markets to fight the negative impacts of the COVID19 pandemic. What are the primary aggregates that are, in essence, new in this balance regarding the standard balance sheet of the Fed?

"""

# ╔═╡ 1a1d2e86-9436-4fe3-8dbd-74509f439a98
md"""
$$\begin{aligned}
& \text {Table 9.1 $-$ Balance Sheet of the Federal Reserve Board: August 11, 2021}\\
&\begin{array}{lr|lr}
\hline \hline \text {\textbf{Assets}} & \text{\$ millions} & \text{\textbf{Liabilities}} & \text{ \$ millions} \\
\hline 
\text{Foreign reserves} & \qquad \quad  38611   & \text{Currency}                & \qquad \quad  2139815\\
\text{Securities}       & \qquad \quad  8028859 & \text{Reserves}                & \qquad \quad  4369073 \\
\text{Loans}            & \qquad \quad  82500   & \text{Reverse Repo Agreements} & \qquad \quad  1267998 \\
\text{Portfolio}        & \qquad \quad  63707   & \text{Government account}      & \qquad \quad  389747 \\
\text{Other assets}     & \qquad \quad  43480   & \text{Other liabilities}       & \qquad \quad  50790 \\
		                &           			& \text{Equity}                  & \qquad \quad  39734 \\
\text{Total assets}     & \qquad \quad 8257157  & \text{Liabilities+Equity}    & \qquad \quad  8257157 \\
\hline
\end{array}
\end{aligned}$$
"""

# ╔═╡ 9c396c2d-9512-4fab-a49f-7883ddb1d98c
md"""
!!! answer 

	Two aggregates are entirely new to the typical balance sheet of a central bank: `Portfolio` on the assets side and `Reverse Repo Agreements` on the liabilities side. 
	
	The Portfolio item reflects the help provided by the Fed to several companies that would collapse only because of the dramatic impacts of the pandemic. The Fed provided financial assistance in exchange for temporarily holding those companies' stock shares. So, the assets held by the Fed increased. In normal circumstances, the Fed does not buy (hold) stocks.
	
	Reverse Repo Agreements is the most interesting new aggregate. In fact, it is not entirely new; what is new is the remarkable value that it assumes in the current Fed's balance sheet. Repos can usually be found on the assets side of the Fed's balance (as part of the item `Securities`) but not on its liabilities side.

	The explanation above will be sufficient at this level of study. However, if students are curious about this specific monetary aggregate, a detailed explanation of their meaning and why they recently increased so much can be found here: [Fed](https://www.newyorkfed.org/markets/domestic-market-operations/monetary-policy-implementation/repo-reverse-repo-agreements). 


"""

# ╔═╡ a0ce2775-792b-4347-8536-02295f35b110


# ╔═╡ 09b4be5f-cb7e-4ffa-bbe2-a4e5b2d4fe66
md"""
# Auxiliary cells (please do not delete)
"""

# ╔═╡ 9fa87191-7c90-4738-a45a-acd929c8bd1b
  TableOfContents()

# ╔═╡ 0e0d036e-2dc8-4eba-b2b3-beeef30fa5c7
begin
      struct TwoColumns{L, R}
    left::L
    right::R
      end
      
      function Base.show(io, mime::MIME"text/html", tc::TwoColumns)
          write(io, """<div style="display: flex;"><div style="flex: 48%;margin-right:2%;">""")
          show(io, mime, tc.left)
          write(io, """</div><div style="flex: 48%;">""")
          show(io, mime, tc.right)
          write(io, """</div></div>""")
      end
end

# ╔═╡ ea9d4a34-4beb-4574-b46f-d1754f955aea
#html"""<style>main {  max-width: 950px; align-self: flex-start;  margin-left: 100px;}"""

# ╔═╡ 28901747-6c9f-446f-a7af-10039813a295
md"### CSS code"

# ╔═╡ a40648b1-66ca-4954-8cb5-1f53e552ef9e
html"""
<style>
	@media screen {
		main {
			margin: 0 auto;
			max-width: 1400px;
    		padding-left: max(100px, 5%);
    		padding-right: max(383px, 10%); 
            # 383px to accomodate TableOfContents(aside=true)
		}
	}
</style>
"""

# ╔═╡ 387535fd-6359-4fa0-9aa7-4945ef1791d7
md"""
#### Supporting cells: exercise 3
"""

# ╔═╡ 26f09352-23fe-4f12-93ce-9298ea95ff61
Fed1 = CSV.read("Assets.csv", delim=";", DataFrame);

# ╔═╡ 98128fbc-f645-4e54-8025-2afe3cbe0c11
period4_1 = MonthlyDate(2003,1):Month(1):MonthlyDate(2022,8);

# ╔═╡ 83b2a77f-c6df-4c0d-aa5d-b82d193b6283
begin
	trace6_1 = scatter(;x = Date.(period4_1), y = Fed1[:,3], 
				name = "MB", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	marker_color = "Blue")
	
	trace6_2 = scatter(;x = Date.(period4_1), y = Fed1[:,2], 
				name = "Assets", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3,
            	marker_color = "Red")

	layout6_2 = Layout(;
		
			title_text = "Total Assets vs Monetary Base in the US (2003--2022): Millions of Dollars", title_x=0.5,
		
			hovermode="x",

			legend=attr( x=0.05, y=0.95),
		
            xaxis = attr(
               title = "Monthly obervations",
               #tickformat = "%Y",
               #hoverformat = "%Y-M%m",
               tick0 = "2003/01/01",
               dtick = "M24" 
			   ),
		
        	#xaxis_range = [Date.(2003), Date.(2023)],
			xaxis_range = ["2002-10-01" , "2022-10-01"],
        	yaxis_title = "Millions of Dollars",
        	#yaxis_range=[-2, 2], 
        	titlefont_size = 16)

	p6_2 = Plot([trace6_1, trace6_2], layout6_2)
end

# ╔═╡ fe63e673-4425-42a9-9f29-98c5f9c088d9
md"""
#### Supporting cells: exercise 6
"""

# ╔═╡ e99b0ed8-53d4-4788-b85d-ecfffc26e538
How_Far= CSV.read("How_Far.csv", DataFrame);

# ╔═╡ 18172ee2-1ffe-4a72-99c7-bef149be5670
period6_0 = MonthlyDate(1990,1):Month(1):MonthlyDate(2023,9);

# ╔═╡ ee1ae95d-28ef-4430-b287-b0eb58ea2a82
period6_1 = MonthlyDate(2000,1):Month(1):MonthlyDate(2023,9);

# ╔═╡ 642da752-5c76-482e-97e5-a9964e894868
begin
	fig6_1 = Plot(Date.(period6_1), [How_Far[:,2] How_Far[:,3] How_Far[:,4] ], line_width = 2, Layout(hovermode ="x", 
	title_text = "Inflation and the Fed funds rate (FFR) for the USA (2000--2022)", title_x=0.5,))
	restyle!(fig6_1, 1:3, name=["Head" "Core" "FFR"], line_color = ["blue" , "red" , "gray"])
	fig6_1
end


# ╔═╡ a8dbf8b7-1f6a-48c1-b8e8-c73062f72277
begin
	fig5_5 = Plot(bar(; x= Date.(period6_1),  y =  How_Far[:,3] - How_Far[:,4]) , Layout(hovermode ="x", 	title_text = "Core inflation minus the Fed funds rate for the USA (2000--2023)", title_x=0.5,))
end

# ╔═╡ 1d7fe1de-5ab9-4861-8f70-9b2ac80b6396
md"""
#### Supporting files: exercise 8
"""

# ╔═╡ 6fc50e20-f49e-4186-851e-416e87f422c9
Oil= CSV.read("Oil_CPI.csv", DataFrame);

# ╔═╡ ef782157-03be-4b91-b2e2-02df422ae736
Plot(Oil[325:end,3] , Oil[325:end,2], mode = "markers", Layout(title_text = "Headline inflation vs Oil prices (1998--2023)", title_x=0.5,xaxis_title = "Anual growth rates of Oil prices",yaxis_title = "Annual headline inflation"))

# ╔═╡ add6d8ca-1752-4163-bd31-046868a28c9e
period9_1 = MonthlyDate(1971,1):Month(1):MonthlyDate(2023,9);

# ╔═╡ 159870f2-589d-42f9-85ad-25c11a4c4366
period9_2 = MonthlyDate(1998,1):Month(1):MonthlyDate(2023,9);

# ╔═╡ f342e1f8-03a6-42d4-b220-6fc8acb98b56
begin
	trace10_5 = scatter(;x = Date.(period9_2), y = Oil[325:end,3], name = "Oil", line_color = "Blue", 
						mode = "lines", line_width="2", marker_size ="6")
	
	
	trace10_6 = scatter(;x = Date.(period9_2), y = Oil[325:end,2], name = "CPI", line_color = "red", 
						mode = "lines", line_width="2", marker_size ="4", 
						yaxis = "y2")
	
	
	layout10_6 = Layout(
		
			title_text = "Oil prices growth rate vs Headline inflation for the US (1998--2023)", title_x=0.5,
		
			hovermode="x",

			legend=attr( x=0.05, y=0.95),
		
			xaxis = attr(
                		title = "Monthly obervations",
                		tickformat = "%Y",
                		#hoverformat = "%Y-M%m",
                		#tick0 = "1989/19/01",
                		#dtick = "M48"
        						),
		
			yaxis1 = attr(
						title = "Oil",
						titlefont_color=  "blue",
						tickfont_color = "blue",
						#overlaying = "y",
						#side = "right",
						#yaxis1_tickvals = [1.5, 2.5, 3.5, 5.5],
						showgrid= false,
						zeroline=false,
    					#dtick = 5.6 / 7
						),
	

	  		yaxis2 = attr(
						title = "CPI",
						titlefont_color=  "red",
						tickfont_color = "red",
						overlaying = "y",
						side = "right",
						showgrid= false,
						zeroline=false,
						#yaxis2_range=[-27 , 27],
    					#dtick = 27 / 3
	  					)
		
						)

	p10_6 = Plot([trace10_5,trace10_6],layout10_6)
end

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
PeriodicalDates = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
PlotlyBase = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoTeachingTools = "661c6b06-c737-4d37-b85c-46df65de6f69"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
StatsBase = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"

[compat]
CSV = "~0.10.4"
DataFrames = "~1.4.1"
HypertextLiteral = "~0.9.5"
PeriodicalDates = "~2.0.0"
PlotlyBase = "~0.8.19"
PlutoPlotly = "~0.4.5"
PlutoTeachingTools = "~0.2.3"
PlutoUI = "~0.7.43"
StatsBase = "~0.34.2"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.10.0"
manifest_format = "2.0"
project_hash = "8e8c5e3034d9c859424644997c5ae0de1d517ec4"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "0f748c81756f2e5e6854298f11ad8b2dfae6911a"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.3.0"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.1"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[deps.BaseDirs]]
git-tree-sha1 = "3e93fcd95fe8db4704e98dbda14453a0bfc6f6c3"
uuid = "18cc8868-cbac-4acf-b575-c8ff214dc66f"
version = "1.2.3"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "679e69c611fff422038e9e21e270c4197d49d918"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.12"

[[deps.CodeTracking]]
deps = ["InteractiveUtils", "UUIDs"]
git-tree-sha1 = "c0216e792f518b39b22212127d4a84dc31e4e386"
uuid = "da1fd8a2-8d9e-5ec2-8556-3022fb5608a2"
version = "1.3.5"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "59939d8a997469ee05c4b4944560a820f9ba0d73"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.4"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "67c1f244b991cad9b0aa4b7540fb758c2488b129"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.24.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "eb7f0f8307f71fac7c606984ea5fb2817275d6e4"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.4"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

    [deps.ColorVectorSpace.weakdeps]
    SpecialFunctions = "276daf66-3868-5448-9aa4-cd146d93841b"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "fc08e5930ee9a4e03f84bfb5211cb54e7769758a"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.10"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "c955881e3c981181362ae4088b35995446298b80"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.14.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.0.5+1"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "Future", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrettyTables", "Printf", "REPL", "Random", "Reexport", "SnoopPrecompile", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "d4f69885afa5e6149d0cab3818491565cf41446d"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.4.4"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "1fb174f0d48fe7d142e1109a10636bc1d14f5ac2"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.17"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates", "Mmap", "Printf", "Test", "UUIDs"]
git-tree-sha1 = "9f00e42f8d99fdde64d40c8ea5d14269a2e2c1aa"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.21"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "335bfdceacc84c5cdf16aadc768aa5ddfc5383cc"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.4"

[[deps.Formatting]]
deps = ["Logging", "Printf"]
git-tree-sha1 = "fb409abab2caf118986fc597ba84b50cbaf00b87"
uuid = "59287772-0a20-5a39-b81b-1366585eb4c0"
version = "0.4.3"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "8b72179abc660bfab5e28472e019392b97d0985c"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.4"

[[deps.InlineStrings]]
deps = ["Parsers"]
git-tree-sha1 = "9cc2baf75c6d09f9da536ddf58eb2f29dedaf461"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[deps.InvertedIndices]]
git-tree-sha1 = "0dc7b50b8d436461be01300fd8cd45aa0274b038"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.0"

[[deps.IrrationalConstants]]
git-tree-sha1 = "630b497eafcc20001bba38a4651b327dcfc491d2"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.2.2"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.JuliaInterpreter]]
deps = ["CodeTracking", "InteractiveUtils", "Random", "UUIDs"]
git-tree-sha1 = "7b762d81887160169ddfc93a47e5fd7a6a3e78ef"
uuid = "aa1ae85d-cabe-5617-a682-6adf51b2e16a"
version = "0.9.29"

[[deps.LaTeXStrings]]
git-tree-sha1 = "50901ebc375ed41dbf8058da26f9de442febbbec"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.3.1"

[[deps.Latexify]]
deps = ["Formatting", "InteractiveUtils", "LaTeXStrings", "MacroTools", "Markdown", "OrderedCollections", "Printf", "Requires"]
git-tree-sha1 = "f428ae552340899a935973270b8d98e5a31c49fe"
uuid = "23fbe1c1-3f47-55db-b15f-69d7ec21a316"
version = "0.16.1"

    [deps.Latexify.extensions]
    DataFramesExt = "DataFrames"
    SymEngineExt = "SymEngine"

    [deps.Latexify.weakdeps]
    DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
    SymEngine = "123dc426-2d89-5057-bbad-38513e3affd8"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.4.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.6.4+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"

[[deps.LogExpFunctions]]
deps = ["DocStringExtensions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "18144f3e9cbe9b15b070288eef858f71b291ce37"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.27"

    [deps.LogExpFunctions.extensions]
    LogExpFunctionsChainRulesCoreExt = "ChainRulesCore"
    LogExpFunctionsChangesOfVariablesExt = "ChangesOfVariables"
    LogExpFunctionsInverseFunctionsExt = "InverseFunctions"

    [deps.LogExpFunctions.weakdeps]
    ChainRulesCore = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
    ChangesOfVariables = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
    InverseFunctions = "3587e190-3f89-42d0-90ee-14403ec27112"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[deps.LoweredCodeUtils]]
deps = ["JuliaInterpreter"]
git-tree-sha1 = "31e27f0b0bf0df3e3e951bfcc43fe8c730a219f6"
uuid = "6f1432cf-f94c-5a45-995e-cdbf5db27b0b"
version = "2.4.5"

[[deps.MIMEs]]
git-tree-sha1 = "65f28ad4b594aebe22157d6fac869786a255b7eb"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "0.1.4"

[[deps.MacroTools]]
deps = ["Markdown", "Random"]
git-tree-sha1 = "2fa9ee3e63fd3a4f7a9a4f4744a52f4856de82df"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.13"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.2+1"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "f66bdc5de519e8f8ae43bdc598782d35a25b1272"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.1.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.1.10"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.23+2"

[[deps.OrderedCollections]]
git-tree-sha1 = "dfdf5519f235516220579f949664f1bf44e741c5"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.6.3"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.PeriodicalDates]]
deps = ["Dates", "Printf", "RecipesBase"]
git-tree-sha1 = "e616941f8093e50a373e7d51875143213f82eca4"
uuid = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
version = "2.0.0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "Serialization", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.10.0"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "56baf69781fc5e61607c3e46227ab17f7040ffa2"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.19"

[[deps.PlutoHooks]]
deps = ["InteractiveUtils", "Markdown", "UUIDs"]
git-tree-sha1 = "072cdf20c9b0507fdd977d7d246d90030609674b"
uuid = "0ff47ea0-7a50-410d-8455-4348d5de0774"
version = "0.0.5"

[[deps.PlutoLinks]]
deps = ["FileWatching", "InteractiveUtils", "Markdown", "PlutoHooks", "Revise", "UUIDs"]
git-tree-sha1 = "8f5fa7056e6dcfb23ac5211de38e6c03f6367794"
uuid = "0ff47ea0-7a50-410d-8455-4348d5de0420"
version = "0.1.6"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "BaseDirs", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "Reexport", "TOML"]
git-tree-sha1 = "fbf637823ec24c5669b1a66f3771c2306f60857c"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.4.5"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoTeachingTools]]
deps = ["Downloads", "HypertextLiteral", "LaTeXStrings", "Latexify", "Markdown", "PlutoLinks", "PlutoUI", "Random"]
git-tree-sha1 = "89f57f710cc121a7f32473791af3d6beefc59051"
uuid = "661c6b06-c737-4d37-b85c-46df65de6f69"
version = "0.2.14"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "71a22244e352aa8c5f0f2adde4150f62368a3f2e"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.58"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "03b4c25b43cb84cee5c90aa9b5ea0a78fd848d2f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.0"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "9e8fed0505b0c15b4c1295fd59ea47b411c019cf"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.2"

[[deps.PrettyTables]]
deps = ["Crayons", "LaTeXStrings", "Markdown", "PrecompileTools", "Printf", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "88b895d13d53b5577fd53379d913b9ab9ac82660"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.3.1"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[deps.RecipesBase]]
deps = ["PrecompileTools"]
git-tree-sha1 = "5c3d09cc4f31f5fc6af001c250bf1278733100ff"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.4"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.Revise]]
deps = ["CodeTracking", "Distributed", "FileWatching", "JuliaInterpreter", "LibGit2", "LoweredCodeUtils", "OrderedCollections", "Pkg", "REPL", "Requires", "UUIDs", "Unicode"]
git-tree-sha1 = "12aa2d7593df490c407a3bbd8b86b8b515017f3e"
uuid = "295af30f-e4ad-537b-8983-00126c2a3abe"
version = "3.5.14"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "0e7508ff27ba32f26cd459474ca2ede1bc10991f"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.1"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[deps.SnoopPrecompile]]
deps = ["Preferences"]
git-tree-sha1 = "e760a70afdcd461cf01a575947738d359234665c"
uuid = "66db9d55-30c0-4569-8b51-7e840670fc0c"
version = "1.0.3"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.SparseArrays]]
deps = ["Libdl", "LinearAlgebra", "Random", "Serialization", "SuiteSparse_jll"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
version = "1.10.0"

[[deps.Statistics]]
deps = ["LinearAlgebra", "SparseArrays"]
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.10.0"

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1ff449ad350c9c4cbc756624d6f8a8c3ef56d3ed"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.7.0"

[[deps.StatsBase]]
deps = ["DataAPI", "DataStructures", "LinearAlgebra", "LogExpFunctions", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "1d77abd07f617c4868c33d4f5b9e1dbb2643c9cf"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.34.2"

[[deps.StringManipulation]]
deps = ["PrecompileTools"]
git-tree-sha1 = "a04cabe79c5f01f4d723cc6704070ada0b9d46d5"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.3.4"

[[deps.SuiteSparse_jll]]
deps = ["Artifacts", "Libdl", "libblastrampoline_jll"]
uuid = "bea87d4a-7f5b-5778-9afe-8cc45184846c"
version = "7.2.1+1"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "LinearAlgebra", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "cb76cf677714c095e535e3501ac7954732aeea2d"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.11.1"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"

[[deps.TranscodingStreams]]
git-tree-sha1 = "54194d92959d8ebaa8e26227dbe3cdefcdcd594f"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.10.3"
weakdeps = ["Random", "Test"]

    [deps.TranscodingStreams.extensions]
    TestExt = ["Test", "Random"]

[[deps.Tricks]]
git-tree-sha1 = "eae1bb484cd63b36999ee58be2de6c178105112f"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.8"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.8.0+1"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.52.0+1"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─ee9154d6-739a-46d6-aa1c-0467d4b49f5a
# ╟─943edb36-acc8-4b2a-94ab-c544f9eb279b
# ╟─1245aef9-18ce-42f5-af82-c2e6cce70798
# ╟─4618a663-f77e-4ccc-9b00-e84dbd5beb34
# ╠═a145e2fe-a903-11eb-160f-df7ea83fa3e6
# ╟─74de9f66-d5b4-4e0f-a75d-5177b1842191
# ╟─72d45107-22d6-4663-a9b8-c540a4c7411b
# ╟─116306a9-5f2d-4b96-9124-584dc9835a8a
# ╟─9877083b-a80c-4069-89e4-84638fe6ecbb
# ╟─0be26b61-30ce-4128-b2b1-b4eb955244d1
# ╟─cfc72d68-3d45-4673-81a2-7032896fbe89
# ╟─be5df6ed-9417-4ed0-862c-8f337b80084e
# ╟─f0a318ef-47b0-4fa3-aeb1-e7d1548fd214
# ╟─b189ebff-1493-4d95-98d2-99d6f55884fe
# ╠═c99c90e1-e91f-4682-a190-495d2cbd0b06
# ╠═98e7e1ad-2964-41e5-9162-cd077978ee15
# ╟─719dbca7-f3f1-4827-bfba-d9813979c42a
# ╟─f83dd4f0-0312-4bb0-98b0-67eeab7177e4
# ╠═11d60c86-9825-4113-883f-66c82bcb29e9
# ╠═bc07b610-26f4-4577-b7cb-0089f08b7a60
# ╟─02126494-1ba6-47f8-8c10-9c4d34e20d57
# ╟─84cee9af-0999-4659-a537-85c89f9182cc
# ╟─c144c3fe-d49f-4243-bdc2-34ba8af88673
# ╟─1a073295-059c-4815-b3bd-450f5f88c8d1
# ╟─99fa722b-bef4-48d7-bd4a-f0459f15eed3
# ╟─f4706381-35df-44dd-aea2-83220a94c9d4
# ╟─8eeea220-e319-45da-a774-7cf1475f2424
# ╟─4f82c4f5-dab7-4ef4-a1c6-bef80ac2fe64
# ╟─71bbd822-914c-4ea6-93c5-0b3ad86000ca
# ╟─10234ea1-6528-4fd2-bdae-3ca5a36e405b
# ╟─76d208b1-80b7-4a8c-96c6-2de15b5e89e8
# ╟─1254cc13-8ac0-468c-bbc5-e838018fa1de
# ╟─e7f794fa-ede6-4391-b703-5488fd739f29
# ╟─15b3a15f-a0a0-4623-b6ec-43dc33bbe1e3
# ╟─d2a12416-9d54-479c-956d-2d8869024727
# ╟─3323975f-e90c-4a39-87d4-2ae9c676e24a
# ╟─a4aecd27-1376-4f74-98b6-10502c93e1d0
# ╟─d865e28a-5db4-4b39-a7e6-353d380b9e1a
# ╟─32c928dd-ca87-4db5-aa28-222e43a7a3cb
# ╟─5d7ad15e-3057-43c8-913c-67f6a71338b7
# ╟─821afa78-c1c7-4f08-a1a4-0ae7682c1c2b
# ╟─c842b2da-1bf5-43db-9bc0-a8af8be64b9c
# ╟─7e7e4292-dfc8-41b4-a5a0-d7646f8fb495
# ╟─83b2a77f-c6df-4c0d-aa5d-b82d193b6283
# ╟─e547d364-216c-4165-982f-1e04c9a0945c
# ╟─f54d0589-0567-4b75-91f2-d9bf223c5fb6
# ╟─2bd8b31a-8fa2-4404-816e-158df0afcc7d
# ╟─e7fd66aa-cb38-4b7f-92af-aa4fd15549b2
# ╠═cd8e9140-5696-4019-b208-d6430e68f978
# ╟─f8a3949e-aae6-471f-8fb4-d536566d6e8a
# ╟─c5363210-26f9-40b1-9a1a-9ac80ffac890
# ╟─a9095027-f033-4253-9269-105e51887424
# ╟─1e9a9875-a58e-4e4c-add2-9d5cd667384f
# ╟─c11ab4d4-281e-435b-9ba6-35ef7f4d24b9
# ╟─1471566f-469f-421b-8671-abc49e040bc1
# ╟─624e0704-a138-4ddd-8086-078d98a84a41
# ╟─fe0e5144-02b9-4df3-84ad-3298a8703baa
# ╟─6e20a099-8d46-4a35-83ce-e62c977a6b3d
# ╟─65da15c3-dcad-4e38-b0aa-bf9dd456285d
# ╟─ced280ff-d8c1-414e-8855-387727d5dfe2
# ╟─74529060-483d-4afe-8d58-155c5f0b4b1e
# ╟─5e6f355c-cb29-416f-8799-0234fbc87f1f
# ╟─f4fe404c-ca57-4682-8a51-3c08abedfb36
# ╟─60e62656-e863-4595-9d94-4e7d0cf6d935
# ╟─88675aac-0d69-4028-aee1-c0e0c22ab5e2
# ╟─642da752-5c76-482e-97e5-a9964e894868
# ╟─08bf1683-0c0d-4c52-a446-6f8be4135aed
# ╟─d08dc62f-8951-4e36-8818-c325316ece23
# ╟─bf337699-7647-4538-85e1-ed2fd9d05269
# ╟─5b1919fb-f5f5-4682-a1e0-1c767825c902
# ╟─a8dbf8b7-1f6a-48c1-b8e8-c73062f72277
# ╟─c93efdd7-1ae9-4989-b5cb-cfdb4347086d
# ╟─8f643bb9-8826-421f-8590-b440c65101eb
# ╟─dca53198-44ed-402e-8a68-401a6006d1f9
# ╟─2b7f9399-1606-4374-abfb-abf9e7ab2e90
# ╟─f342e1f8-03a6-42d4-b220-6fc8acb98b56
# ╟─ef782157-03be-4b91-b2e2-02df422ae736
# ╟─0223f27f-532b-4b75-af02-817fe5f44ed6
# ╟─b04f7af0-7e7c-45a5-ad82-78e866348567
# ╟─1a8a1ae4-6ecd-4a91-a482-3024ca11fb98
# ╟─69621e4b-5163-40a4-a70c-429d8afe6ff5
# ╟─01875faf-3231-4bb8-899d-0dd4e569ddc9
# ╟─63efdb6e-dff1-420f-a99d-90cece86bac2
# ╟─1a1d2e86-9436-4fe3-8dbd-74509f439a98
# ╟─9c396c2d-9512-4fab-a49f-7883ddb1d98c
# ╟─a0ce2775-792b-4347-8536-02295f35b110
# ╟─09b4be5f-cb7e-4ffa-bbe2-a4e5b2d4fe66
# ╟─9fa87191-7c90-4738-a45a-acd929c8bd1b
# ╟─0e0d036e-2dc8-4eba-b2b3-beeef30fa5c7
# ╟─ea9d4a34-4beb-4574-b46f-d1754f955aea
# ╟─28901747-6c9f-446f-a7af-10039813a295
# ╟─a40648b1-66ca-4954-8cb5-1f53e552ef9e
# ╟─387535fd-6359-4fa0-9aa7-4945ef1791d7
# ╟─26f09352-23fe-4f12-93ce-9298ea95ff61
# ╟─98128fbc-f645-4e54-8025-2afe3cbe0c11
# ╟─fe63e673-4425-42a9-9f29-98c5f9c088d9
# ╟─e99b0ed8-53d4-4788-b85d-ecfffc26e538
# ╟─18172ee2-1ffe-4a72-99c7-bef149be5670
# ╟─ee1ae95d-28ef-4430-b287-b0eb58ea2a82
# ╟─1d7fe1de-5ab9-4861-8f70-9b2ac80b6396
# ╟─6fc50e20-f49e-4186-851e-416e87f422c9
# ╟─add6d8ca-1752-4163-bd31-046868a28c9e
# ╟─159870f2-589d-42f9-85ad-25c11a4c4366
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
